# Copyright (c) 2000-2009, JPackage Project
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the JPackage Project nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

%define with()          %{expand:%%{?with_%{1}:1}%%{!?with_%{1}:0}}
%define without()       %{expand:%%{?with_%{1}:0}%%{!?with_%{1}:1}}
%define bcond_with()    %{expand:%%{?_with_%{1}:%%global with_%{1} 1}}
%define bcond_without() %{expand:%%{!?_without_%{1}:%%global with_%{1} 1}}

%bcond_with bootstrap
%bcond_with gcj_support
%bcond_with maven
%bcond_without repolib

%define repodir %{_javadir}/repository.jboss.com/apache-logging/%{version}-brew
%define repodirlib %{repodir}/lib
%define repodirsrc %{repodir}/src

%if %with gcj_support
%define gcj_support 1
%else
%define gcj_support 0
%endif

%define base_name  logging
%define short_name commons-%{base_name}
%define section    free

Name:           jakarta-%{short_name}
Version:        1.1.1
Release:        1%{?dist}
Epoch:          0
Summary:        Jakarta Commons Logging Package
License:        ASL 2.0
Group:          Development/Libraries/Java
URL:            http://commons.apache.org/%{base_name}/
Source0:        http://www.apache.org/dist/jakarta/commons/logging/source/commons-logging-1.1.1-src.tar.gz
Source1:        pom-maven2jpp-depcat.xsl
Source2:        pom-maven2jpp-newdepmap.xsl
Source3:        pom-maven2jpp-mapdeps.xsl
Source4:        commons-logging-1.1.1-jpp-depmap.xml
Source5:        http://mirrors.ibiblio.org/pub/mirrors/maven2/commons-logging/commons-logging/1.1.1/commons-logging-1.1.1.pom
Source6:        commons-logging-tomcat6-build.xml
Source7:        commons-logging-component-info.xml
Source8:	jakarta-commons-logging-upload.sh
Patch0:         commons-logging-1.1.1-build_xml.patch
Requires(post): jpackage-utils >= 0:1.7.2
Requires(postun): jpackage-utils >= 0:1.7.2
BuildRequires:  jpackage-utils >= 0:1.7.2
BuildRequires:  ant >= 0:1.6
BuildRequires:  maven-deploy-plugin
%if %with maven
BuildRequires:  maven >= 0:1.1
BuildRequires:  maven-plugins-base
BuildRequires:  maven-plugin-jdepend
BuildRequires:  maven-plugin-jxr
BuildRequires:  maven-plugin-license
BuildRequires:  maven-plugin-linkcheck
BuildRequires:  maven-plugin-xdoc
BuildRequires:  saxon
BuildRequires:  saxon-scripts
%endif
%if %without bootstrap
BuildRequires:  ant-junit
BuildRequires:  junit
BuildRequires:  avalon-logkit
BuildRequires:  avalon-framework
%endif
BuildRequires:  log4j
BuildRequires:  servletapi4
%if %{gcj_support}
BuildRequires:  java-gcj-compat-devel
%else
BuildArch:      noarch
%endif
BuildRoot:      %{_tmppath}/%{name}-%{version}-%{release}-root

%description
The commons-logging package provides a simple, component oriented
interface (org.apache.commons.logging.Log) together with wrappers for
logging systems. The user can choose at runtime which system they want
to use. In addition, a small number of basic implementations are
provided to allow users to use the package standalone. 
commons-logging was heavily influenced by Avalon's Logkit and Log4J. The
commons-logging abstraction is meant to minimixe the differences between
the two, and to allow a developer to not tie himself to a particular
logging implementation.

%package javadoc
Summary:        Javadoc for %{name}
Group:          Development/Documentation

%description javadoc
%{summary}.

%if %with maven
%package manual
Summary:        Documents for %{name}
Group:          Development/Documentation

%description manual
%{summary}.
%endif

%package tomcat6
Summary:        Commons Logging dependency for Tomcat6
Group:          Development/Libraries/Java

%description tomcat6
Commons Logging dependency for Tomcat6.

%if %with repolib
%package repolib
Summary:        Artifacts to be uploaded to a repository library
Group:          Development/Libraries/Java

%description repolib
Artifacts to be uploaded to a repository library.
This package is not meant to be installed but so its contents
can be extracted through rpm2cpio.
%endif
# -----------------------------------------------------------------------------

%prep
%setup -q -n %{short_name}-%{version}-src
%patch0 -p1
%{__cp} -p %{SOURCE6} commons-logging-tomcat6-build.xml
%{__perl} -pi -e 's/\r$//g' LICENSE.txt RELEASE-NOTES.txt
%{__cp} -p %{SOURCE0} commons-logging-%{version}-src.tar.gz
%{__gzip} -dc %{SOURCE0} > commons-logging-%{version}-src.tar

find . -name "*.jar" -exec rm -f {} \;

# -----------------------------------------------------------------------------

%build
export CLASSPATH=
export OPT_JAR_LIST="`%{__cat} %{_sysconfdir}/ant.d/junit`"
cat > build.properties <<EOBM
component.version=%{version}
junit.jar=$(build-classpath junit)
log4j.jar=$(build-classpath log4j)
log4j12.jar=$(build-classpath log4j)
%if %without bootstrap
logkit.jar=$(build-classpath avalon-logkit)
avalon-framework.jar=$(build-classpath avalon-framework)
%endif
servletapi.jar=$(build-classpath servletapi4)
EOBM
#  <property name="junit.jar"               value="junit-3.8.1.jar"/>
#  <property name="log4j12.jar"             value="log4j-1.2.12.jar"/>
#  <property name="log4j13.jar"             value="log4j-1.3.0.jar"/>
#  <property name="logkit.jar"              value="logkit-1.0.1.jar"/>
#  <property name="avalon-framework.jar"    value="avalon-framework-4.1.3.jar"/>
#  <property name="servletapi.jar"          value="servletapi-2.3.jar"/>

%if %without bootstrap
%if %with maven
export DEPCAT=$(pwd)/commons-logging-1.1-depcat.new.xml
echo '<?xml version="1.0" standalone="yes"?>' > $DEPCAT
echo '<depset>' >> $DEPCAT
for p in $(find . -name project.xml); do
    pushd $(dirname $p)
    %{_bindir}/saxon project.xml %{SOURCE1} >> $DEPCAT
    popd
done
echo >> $DEPCAT
echo '</depset>' >> $DEPCAT
%{_bindir}/saxon $DEPCAT %{SOURCE2} > commons-logging-1.1-depmap.new.xml

for p in $(find . -name project.xml); do
    pushd $(dirname $p)
    cp project.xml project.xml.orig
    %{_bindir}/saxon -o project.xml project.xml.orig %{SOURCE3} map=%{SOURCE4}
    popd
done

mkdir -p .maven/repository/JPP/plugins/
pushd .maven/repository/JPP/plugins/
ln -sf /usr/share/maven/plugins/maven-xdoc-plugin-*.jar maven-xdoc-plugin.jar
popd

export MAVEN_HOME_LOCAL=$(pwd)/.maven
maven \
        -Dmaven.repo.remote=file:/usr/share/maven/repository \
        -Dmaven.home.local=$MAVEN_HOME_LOCAL \
        jar:install javadoc:generate xdoc:transform

%else
ant \
        -Djunit.jar=$(build-classpath junit) \
        -Dlogkit.jar=$(build-classpath excalibur/avalon-logkit) \
        -Davalon-framework.jar=$(build-classpath excalibur/avalon-framework) \
        -Dservletapi.jar=$(build-classpath servletapi4) \
        compile.tests javadoc
%endif
## FIXME: There are failures with gcj. Ignore them for now.
%if %{gcj_support}
  ant -Dtest.failonerror=false \
        -Djunit.jar=$(build-classpath junit) \
        -Dlogkit.jar=$(build-classpath excalibur/avalon-logkit) \
        -Davalon-framework.jar=$(build-classpath excalibur/avalon-framework) \
        -Dservletapi.jar=$(build-classpath servletapi4) \
    test
%else
  ant  \
        -Dtest.failonerror=false \
        -Djunit.jar=$(build-classpath junit) \
        -Dlogkit.jar=$(build-classpath excalibur/avalon-logkit) \
        -Davalon-framework.jar=$(build-classpath excalibur/avalon-framework) \
        -Dservletapi.jar=$(build-classpath servletapi4) \
    test
%endif
%else
        ant compile-only javadoc
%endif

export CLASSPATH=
export OPT_JAR_LIST=:
%{ant} \
        -Dcommons-logging-version=%{version} \
        -Dcommons-logging-source=commons-logging-%{version}-src.tar \
        -Dtomcat.extras=`pwd`/commons-logging-tomcat6 \
        -Dtomcat.classes=`pwd`/target/classes \
    -f commons-logging-tomcat6-build.xml

%install
rm -rf $RPM_BUILD_ROOT

# jars
install -d -m 755 $RPM_BUILD_ROOT%{_javadir}

install -p -m 644 target/%{short_name}-%{version}.jar \
                $RPM_BUILD_ROOT%{_javadir}/%{name}-%{version}.jar
install -p -m 644 target/%{short_name}-api-%{version}.jar \
                $RPM_BUILD_ROOT%{_javadir}/%{name}-api-%{version}.jar
install -p -m 644 target/%{short_name}-adapters-%{version}.jar \
                $RPM_BUILD_ROOT%{_javadir}/%{name}-adapters-%{version}.jar

# tomcat6 jars
%{__cp} -p commons-logging-tomcat6/tomcat-juli.jar %{buildroot}%{_javadir}/%{name}-tomcat-juli-%{version}.jar
%{__cp} -p commons-logging-tomcat6/tomcat-juli-adapters.jar %{buildroot}%{_javadir}/%{name}-tomcat-juli-adapters-%{version}.jar

(cd $RPM_BUILD_ROOT%{_javadir} && for jar in *-%{version}*; do ln -sf ${jar} `echo $jar| sed "s|%{name}|%{short_name}|g"`; done)

(cd %{buildroot}%{_javadir} && %{__ln_s} %{name}-tomcat-juli-adapters-%{version}.jar tomcat-juli-adapters-%{version}.jar)

(cd $RPM_BUILD_ROOT%{_javadir} && for jar in *-%{version}*; do ln -sf ${jar} `echo $jar| sed "s|-%{version}||g"`; done)

# poms
install -d -m 755 $RPM_BUILD_ROOT%{_datadir}/maven2/poms
install -pm 644 %{SOURCE5} \
    $RPM_BUILD_ROOT%{_datadir}/maven2/poms/JPP-%{short_name}.pom
%add_to_maven_depmap %{short_name} %{short_name} %{version} JPP %{short_name}

# javadoc
install -d -m 755 $RPM_BUILD_ROOT%{_javadocdir}/%{name}-%{version}
%if %without bootstrap
%if %with maven
cp -pr target/docs/apidocs/* $RPM_BUILD_ROOT%{_javadocdir}/%{name}-%{version}
rm -rf target/docs/apidocs
%else
cp -pr dist/docs/api/* $RPM_BUILD_ROOT%{_javadocdir}/%{name}-%{version}
%endif
%else
cp -pr dist/docs/api/* $RPM_BUILD_ROOT%{_javadocdir}/%{name}-%{version}
%endif
ln -s %{name}-%{version} $RPM_BUILD_ROOT%{_javadocdir}/%{name}

%if %without bootstrap
%if %with maven
# manual
install -d -m 755 $RPM_BUILD_ROOT%{_docdir}/%{name}-%{version}/site
cp -pr target/docs/* $RPM_BUILD_ROOT%{_docdir}/%{name}-%{version}/site
%endif
%endif

%if %with repolib
%{__install} -d -m 755 %{buildroot}%{repodir}
%{__install} -d -m 755 %{buildroot}%{repodirlib}
%{__install} -p -m 644 %{SOURCE7} %{buildroot}%{repodir}/component-info.xml
tag=`/bin/echo %{name}-%{version}-%{release} | %{__sed} 's|\.|_|g'`
%{__sed} -i "s/@TAG@/$tag/g" %{buildroot}%{repodir}/component-info.xml
%{__sed} -i "s/@VERSION@/%{version}-brew/g" %{buildroot}%{repodir}/component-info.xml
%{__install} -d -m 755 %{buildroot}%{repodirsrc}
%{__install} -p -m 644 %{SOURCE0} %{buildroot}%{repodirsrc}
%{__install} -p -m 644 %{SOURCE1} %{buildroot}%{repodirsrc}
%{__install} -p -m 644 %{SOURCE2} %{buildroot}%{repodirsrc}
%{__install} -p -m 644 %{SOURCE3} %{buildroot}%{repodirsrc}
%{__install} -p -m 644 %{SOURCE4} %{buildroot}%{repodirsrc}
%{__install} -p -m 644 %{SOURCE5} %{buildroot}%{repodirsrc}
%{__install} -p -m 644 %{SOURCE6} %{buildroot}%{repodirsrc}
%{__install} -p -m 644 %{PATCH0} %{buildroot}%{repodirsrc}
%{__cp} -p %{buildroot}%{_javadir}/%{name}-%{version}.jar %{buildroot}%{repodirlib}/%{short_name}.jar
%{__cp} -p %{buildroot}%{_datadir}/maven2/poms/JPP-%{short_name}.pom %{buildroot}%{repodirlib}/%{short_name}.pom
%{__install} -p -m 755 %{SOURCE8} %{buildroot}%{repodir}/upload.sh
pushd %{buildroot}%{repodir}
./upload.sh %{buildroot}/usr/share/java/repository.jboss.com/maven2-brew/
popd
%endif

%if %{gcj_support}
%{_bindir}/aot-compile-rpm
%endif

%clean
rm -rf $RPM_BUILD_ROOT

%post
%update_maven_depmap
%if %{gcj_support}
if [ -x %{_bindir}/rebuild-gcj-db ]
then
  %{_bindir}/rebuild-gcj-db
fi
%endif

%postun
%update_maven_depmap
%if %{gcj_support}
if [ -x %{_bindir}/rebuild-gcj-db ]
then
  %{_bindir}/rebuild-gcj-db
fi
%endif

%if %{gcj_support}
%post tomcat6
if [ -x %{_bindir}/rebuild-gcj-db ]
then
  %{_bindir}/rebuild-gcj-db
fi
%endif

%if %{gcj_support}
%postun tomcat6
if [ -x %{_bindir}/rebuild-gcj-db ]
then
  %{_bindir}/rebuild-gcj-db
fi
%endif

%files
%defattr(0644,root,root,0755)
%doc LICENSE.txt PROPOSAL.html RELEASE-NOTES.txt STATUS.html
%{_javadir}*/%{name}-%{version}.jar
%{_javadir}*/%{name}.jar
%{_javadir}*/%{short_name}-%{version}.jar
%{_javadir}*/%{short_name}.jar
%{_javadir}*/%{name}-adapters-%{version}.jar
%{_javadir}*/%{name}-adapters.jar
%{_javadir}*/%{short_name}-adapters-%{version}.jar
%{_javadir}*/%{short_name}-adapters.jar
%{_javadir}*/%{name}-api-%{version}.jar
%{_javadir}*/%{name}-api.jar
%{_javadir}*/%{short_name}-api-%{version}.jar
%{_javadir}*/%{short_name}-api.jar
%{_datadir}/maven2/poms/*
%config(noreplace) %{_mavendepmapfragdir}/*
%if %{gcj_support}
%dir %{_libdir}/gcj/%{name}
%attr(-,root,root) %{_libdir}/gcj/%{name}/%{name}-%{version}.jar.db
%attr(-,root,root) %{_libdir}/gcj/%{name}/%{name}-%{version}.jar.so
%endif

%files javadoc
%defattr(0644,root,root,0755)
%{_javadocdir}/%{name}-%{version}
%{_javadocdir}/%{name}

%if %without bootstrap
%if %with maven
%files manual
%defattr(0644,root,root,0755)
%doc %{_docdir}/%{name}-%{version}/site
%endif
%endif

%files tomcat6
%defattr(0644,root,root,0755)
%{_javadir}*/%{name}-tomcat-juli-%{version}.jar
%{_javadir}*/%{name}-tomcat-juli.jar
%{_javadir}*/%{short_name}-tomcat-juli-%{version}.jar
%{_javadir}*/%{short_name}-tomcat-juli.jar
%{_javadir}*/%{name}-tomcat-juli-adapters-%{version}.jar
%{_javadir}*/%{name}-tomcat-juli-adapters.jar
%{_javadir}*/%{short_name}-tomcat-juli-adapters-%{version}.jar
%{_javadir}*/%{short_name}-tomcat-juli-adapters.jar
%{_javadir}*/tomcat-juli-adapters-%{version}.jar
%{_javadir}*/tomcat-juli-adapters.jar
%if %{gcj_support}
%attr(-,root,root) %{_libdir}/gcj/%{name}/%{name}-tomcat-juli-%{version}.jar.db
%attr(-,root,root) %{_libdir}/gcj/%{name}/%{name}-tomcat-juli-%{version}.jar.so
%attr(-,root,root) %{_libdir}/gcj/%{name}/%{name}-tomcat-juli-adapters-%{version}.jar.db
%attr(-,root,root) %{_libdir}/gcj/%{name}/%{name}-tomcat-juli-adapters-%{version}.jar.so
%endif

%if %with repolib
%files repolib
%defattr(0644,root,root,0755)
%{_javadir}*/repository.jboss.com
%endif

%changelog
* Tue Dec 21 2010 Fernando Nasser <fnasser@redhat.com> - 0:1.1.1-1
- Rebuild for MEAD loading

* Fri Nov 19 2010 Yiqiao Wang <yiwang@redhat.com> - 0:1.1.1-0.4.1.1
- Add upload.sh to repolib
- Fix %%files for signed jars
- Add content of maven2-brew
- Add the line that removes binaries

* Mon May 17 2010 Martha Benitez <mbenitez@redhat.com> - 0:1.1.1-0.4.1
- rebuilt

* Wed Apr 21 2010 Weinan Li <weli@redhat.com> 0:1.1.1-0.4
- Change package excalibur-avalon-logkit to avalon-logkit and excalibur-avalon-framework to avalon-framework in BRs, fix its jar's location
- Rebuild for generating signed jars

* Wed Dec 09 2009 David Walluck <dwalluck@redhat.com> 0:1.1.1-0.3
- add repolib

* Mon Feb 09 2009 David Walluck <dwalluck@redhat.com> 0:1.1.1-0.2
- update pom

* Thu Feb 05 2009 David Walluck <dwalluck@redhat.com> 0:1.1.1-0.1
- 1.1.1
- add tomcat6 subpackage

* Thu Jul 17 2008 David Walluck <dwalluck@redhat.com> 0:1.1-8
- fix excalibur-avalon-framework symlinks

* Sun Jun 22 2008 David Walluck <dwalluck@redhat.com> 0:1.1-7.jpp5
- build with ant by default

* Sun Jun 22 2008 David Walluck <dwalluck@redhat.com> 0:1.1-6.jpp5
- don't own %%{_mavendepmapfragdir}
- remove javadoc scriptlets

* Sun Jun 22 2008 Fernando Nasser <fnasser@redhat.com> - 0:1.1-5
- Add missing BR for maven-lugin-xdoc

* Mon Nov 19 2007 Ralph Apel <r.apel at r-apel.de> - 0:1.1-4jpp
- Fix depmap fragments and poms

* Fri Aug 31 2007 Ralph Apel <r.apel at r-apel.de> - 0:1.1-3jpp
- Make Vendor, Distribution based on macro
- Add poms and depmap frags
- BR some maven plugins
- Move manual to ./site

* Mon Feb 12 2007 Ralph Apel <r.apel at r-apel.de> - 0:1.1-2jpp
- Add bootstrap option to omit avalon BRs and support, as well as tests
- Add missing BR for ant-junit when not bootstrapping
- Fix to build javadoc without maven

* Wed Sep 27 2006 Ralph Apel <r.apel at r-apel.de> - 0:1.1-1jpp
- Update to 1.1
- Build with maven by default
- Create option --without maven to build with ant

* Thu Aug 10 2006 Deepak Bhole <dbhole@redhat.com> 0:1.0.4-6jpp
- Added missing requirements.

* Wed Jul 19 2006 Deepak Bhole <dbhole@redhat.com> 0:1.0.4-5jpp
- Remove name/release/version defines as applicable.

* Mon Jul 17 2006 Deepak Bhole <dbhole@redhat.com> 0:1.0.4-4jpp
- Added conditional native compiling.

* Thu Mar 30 2006 Ralph Apel <r.apel at r-apel.de> - 0:1.0.4-3jpp
- Replace avalon-logkit with new excalibur-avalon-logkit

* Sun Aug 23 2004 Randy Watler <rwatler at finali.com> - 0:1.0.4-2jpp
- Rebuild with ant-1.6.2

* Thu Jun 24 2004 Kaj J. Niemi <kajtzu@fi.basen.net> - 0:1.0.4-1jpp
- Update to 1.0.4 (tomcat 5.0.27 wants it)
- Drop Patch #0 (jakarta-commons-logging-noclasspath.patch), unnecessary
 
* Tue Jun 17 2003 Ville Skyttä <ville.skytta at iki.fi> - 0:1.0.3-4jpp
- Drop log4j requirement and manifest Class-Path.
- Run unit tests during build.
- Nuke spurious subdir in -javadoc package.
- Some spec file cleanups.

* Fri May 09 2003 David Walluck <david@anti-microsoft.org> 0:1.0.3-3jpp
- update for JPackage 1.5

* Fri Mar 11 2003 Henri Gomez <hgomez@users.sourceforge.net> 1.0.3-2jpp
- update spec to respect JPP 1.5 policy

* Wed Mar 09 2003 Henri Gomez <hgomez@users.sourceforge.net> 1.0.3-1jpp
- 1.0.3
- Built with IBM SDK 1.4 to have JDK 1.4 support compiled in

* Thu Feb 27 2003 Henri Gomez <hgomez@users.sourceforge.net> 1.0.2-2jpp
- fix ASF license and add packager tag

* Mon Sep 30 2002 Henri Gomez <hgomez@users.sourceforge.net> 1.0.2-1jpp
- 1.0.2

* Tue Aug 20 2002 Henri Gomez <hgomez@users.sourceforge.net> 1.0.1-1jpp
- 1.0.1

* Fri Jul 12 2002 Henri Gomez <hgomez@users.sourceforge.net> 1.0-5jpp
- remove Requires logkit (commons-log could works with log4j, jdk1.4)

* Mon Jun 10 2002 Henri Gomez <hgomez@users.sourceforge.net> 1.0-4jpp
- use sed instead of bash 2.x extension in link area to make spec compatible
  with distro using bash 1.1x

* Mon Jun 10 2002 Henri Gomez <hgomez@users.sourceforge.net> 1.0-3jpp
- built with Sun JDK 1.4.0_01 to have JDK 1.4 support compiled in

* Fri Jun 07 2002 Henri Gomez <hgomez@users.sourceforge.net> 1.0-2jpp
- added short names in %%{javadir}, as does jakarta developpers 

* Mon May 06 2002 Guillaume Rousse <guillomovitch@users.sourceforge.net> 1.0-1jpp 
- fist jpp release
